"""
Class for pushing metrics to Prometheus metrics cache (pushgateway)
"""
import socket
import logging
from prometheus_client import push_to_gateway
from nubium_utils.general_runtime_vars import env_vars

LOGGER = logging.getLogger(__name__)


class MetricsPusher:
    """
    Pushes metrics to a prometheus pushgateway
    """

    def __init__(self,
                 job=None,
                 metrics_service_name=None,
                 metrics_service_port=None,
                 metrics_pod_port=None):
        """
        Initializes metrics pusher

        :param job: (str) Unique name of running application
        :param metrics_service_name: (str) host name of metrics service
        :param metrics_service_port: (sr) port of metrics service
        :param metrics_pod_port: (str) port for metrics cache on individual pod
        """

        if not job:
            job = env_vars()['HOSTNAME']
        if not metrics_service_name:
            metrics_service_name = env_vars()['METRICS_SERVICE_NAME']
        if not metrics_service_port:
            metrics_service_port = env_vars()['METRICS_SERVICE_PORT']
        if not metrics_pod_port:
            metrics_pod_port = env_vars()['METRICS_POD_PORT']

        self.job = job
        self.metrics_service_name = metrics_service_name
        self.metrics_service_port = metrics_service_port
        self.metrics_pod_port = metrics_pod_port

        self.metrics_pod_ips = None

    def set_metrics_pod_ips(self):
        """
        Queries metrics service for gateway IP addresses

        A single Kubernetes service redirects to multiple IP addresses for
        redundant Prometheus pushgateways.
        :return: None
        """
        socket_info_list = socket.getaddrinfo(
            self.metrics_service_name, self.metrics_service_port)
        self.metrics_pod_ips = {f'{result[-1][0]}:{self.metrics_pod_port}'
                                for result in socket_info_list}
        LOGGER.debug(f'Set gateway addresses: {self.metrics_pod_ips}')

    def push_metrics(self, registry):
        for gateway in self.metrics_pod_ips:
            try:
                push_to_gateway(gateway, job=self.job, registry=registry, timeout=15)
            except Exception as error:
                LOGGER.error(f'Failed to push to pushgateway {gateway}\n{error}')



# confluent-kafka runtime environment variables

from os import environ
from nubium_utils import general_runtime_vars
from nubium_utils.env_var_generator import env_vars_creator


def default_env_vars():
    """
    Environment variables that have defaults if not specified.
    """
    return {
        'POLL_TIMEOUT': environ.get('POLL_TIMEOUT', '10'),

        # All required below if using "retry" logic
        'CONSUME_TOPICS': environ.get('CONSUME_TOPICS', ''),
        'TIMESTAMP_OFFSET_MINUTES': environ.get('TIMESTAMP_OFFSET_MINUTES', '0'),
        'RETRY_COUNT_MAX': environ.get('RETRY_COUNT_MAX', '0'),
        'PRODUCE_RETRY_TOPICS': environ.get('PRODUCE_RETRY_TOPICS', ''),
        'PRODUCE_FAILURE_TOPICS': environ.get('PRODUCE_FAILURE_TOPICS', ''),
        'MESSAGE_BATCH_MAX_MB': str(int(environ.get('MESSAGE_BATCH_MAX_MB', '4')) * 2**20),
        'MESSAGE_TOTAL_MAX_MB': str(int(environ.get('MESSAGE_TOTAL_MAX_MB', '5')) * 2**20)}


def required_env_vars():
    """
    Environment variables that require a value (aka no default specified).
    """
    return {}


def all_env_vars():
    return {
        **general_runtime_vars.all_env_vars(),
        **default_env_vars(),
        **required_env_vars()
    }


env_vars = env_vars_creator(all_env_vars)



import unittest
from unittest import mock
from unittest.mock import patch

from nubium_utils.faust_utils import FaustAppWrapper
from nubium_utils.faust_utils import InstrumentedApp
from nubium_utils.faust_utils import get_ssl_context


class ImplementedAppWrapper(FaustAppWrapper):

    def _init_agents(self):
        pass

    def _init_records(self):
        pass

    def _init_serializers(self):
        pass

    def _init_tables(self):
        pass

    def _init_topics(self):
        pass


class TestFaustAppWrapper(unittest.TestCase):
    """
    Tests the app wrapper
    """

    def setUp(self) -> None:
        self.env_patch = patch.dict(
            'os.environ',
            {"DO_METRICS_PUSHING": "False"}
        )
        self.env_patch.start()

    def tearDown(self) -> None:
        self.env_patch.stop()

    def test_app_creation(self):

        test_faust_config = {
            'id': 'test-id',
            'broker': 'kafka://test',
            'store': "memory://"
        }
        mock_monitor = mock.MagicMock()

        app_wrapper = ImplementedAppWrapper(test_faust_config, avro_client=None, metrics_manager=mock_monitor)
        assert app_wrapper.app.__class__ == InstrumentedApp

    def test_add_input_topic(self):

        test_faust_config = {
            'id': 'test-id',
            'broker': 'kafka://test',
            'store': "memory://"
        }
        mock_monitor = mock.MagicMock()
        mock_serializer = mock.MagicMock()

        app_wrapper = ImplementedAppWrapper(test_faust_config, avro_client=None, metrics_manager=mock_monitor)
        faust_topic = app_wrapper.add_input_topic('test_topic', mock_serializer)

        # Assert that method returns a faust_topic
        assert faust_topic.get_topic_name() == 'test_topic_Internal_test-id'

        # Assert that a input_topic is added
        input_topic = app_wrapper.input_topics['test_topic']

        # Assert that a converter agent was added
        converter_agent = app_wrapper.app.agents.data['test_topic_agent_internal']
        assert converter_agent.channel == input_topic


class TestFaustHelpers(unittest.TestCase):

    def test_no_ssl_context(self):
        """
        get_ssl_context returns None when `USE_SSL` env variable is False
        """
        with mock.patch('nubium_utils.faust_utils.helpers.env_vars', {'USE_SSL': 'False'}):
            output = get_ssl_context()
            assert output == {}

    def test_get_ssl_context(self):
        """
        SSL context configures from env variables when `USE_SSL` is True
        """
        env_vars_patch = {
            'USE_SSL': 'True',
            'SSL_CA_LOCATION': 'test-ca-location',
            'SSL_CERTIFICATE_LOCATION': 'test',
            'SSL_KEY_LOCATION': 'test'}
        with mock.patch('nubium_utils.faust_utils.helpers.env_vars', env_vars_patch):
            with mock.patch('nubium_utils.faust_utils.helpers.ssl') as ssl_patch:
                output = get_ssl_context()['broker_credentials']
                ssl_patch.create_default_context.assert_called_once()
                output.load_cert_chain.assert_called_once()



import sys
import logging
from .general_runtime_vars import default_env_vars


def _init_handler():
    handler = logging.StreamHandler(sys.stdout)
    handler.setLevel(_get_loglevel())

    handler.setFormatter(
        logging.Formatter(
            "{process} - {filename:15} - {asctime} - {name} - {levelname} - {message}",
            style="{"
        )
    )
    return handler


def _get_loglevel():
    return default_env_vars()['LOGLEVEL']


def init_logger(name='nubium-utils'):
    logger = logging.getLogger(name)
    logger.setLevel(_get_loglevel())
    logger.addHandler(_init_handler())
    logger.propagate = False
    return logger



# runtime environment variables used by both confluent-kafka and faust implementations

from os import environ
from nubium_utils import general_runtime_vars
from nubium_utils.env_var_generator import env_vars_creator


def default_env_vars():
    """
    Environment variables that have defaults if not specified.
    """
    return {
        'FAUST_DATADIR_BASE': environ.get('FAUST_DATADIR_BASE', '/opt/app-root/data/'),
        'USE_ROCKSDB': environ.get('USE_ROCKSDB', 'true'),
        'KIWF_ERROR_STRINGS': environ.get('KIWF_ERROR_STRINGS', 'StaleMetadata,COMMIT OFFSET NOT ADVANCING'),
        'KIWF_LOG_FILEPATH': environ.get('KIWF_LOG_FILEPATH', '/opt/app-root/log/faust-log-file.log'),

        # faust direct runtime configs
        'TOPIC_REPLICATION_FACTOR': environ.get('TOPIC_REPLICATION_FACTOR', '5'),
        'CONSUMER_AUTO_OFFSET_RESET': environ.get('CONSUMER_AUTO_OFFSET_RESET', 'latest'),
        'STREAM_RECOVERY_DELAY': environ.get('STREAM_RECOVERY_DELAY', '10'),
        'PROCESSING_GUARANTEE': environ.get('PROCESSING_GUARANTEE', 'exactly_once'),
        'F_BLOCKING_TIMEOUT': environ.get('F_BLOCKING_TIMEOUT', '30.0'),
        'F_FORCE_BLOCKING_TIMEOUT': environ.get('F_FORCE_BLOCKING_TIMEOUT', '1')
    }


def required_env_vars():
    """
    Environment variables that require a value (aka no default specified).
    """
    return {
        # faust direct runtime configs
        'TOPIC_PARTITIONS': environ['TOPIC_PARTITIONS']
    }


def all_env_vars():
    return {
        **general_runtime_vars.all_env_vars(),
        **default_env_vars(),
        **required_env_vars()
    }


env_vars = env_vars_creator(all_env_vars)



import ssl
import logging
from logging.handlers import RotatingFileHandler
from .faust_runtime_vars import env_vars


LOGGER = logging.getLogger(__name__)


def get_ssl_context():
    """
    Constructs SSL context based on environment variables

    If environment variables are missing,
    then no ssl context is returned so the app can run in local mode
    :returns: Properly formatted SSL context or None
    """

    if env_vars()['USE_SSL'].lower() == 'true':
        ca_file = env_vars()['SSL_CA_LOCATION']
        cert_file = env_vars()["SSL_CERTIFICATE_LOCATION"]
        key_file = env_vars()["SSL_KEY_LOCATION"]
        ssl_context = ssl.create_default_context(
            purpose=ssl.Purpose.SERVER_AUTH,
            cafile=ca_file
        )
        ssl_context.load_cert_chain(
            certfile=cert_file,
            keyfile=key_file
        )
        LOGGER.info('SSL encryption ENABLED')
        return {'broker_credentials': ssl_context}

    else:
        LOGGER.info('SSL encryption DISABLED')
        return {}


def get_data_store():
    try:
        store = env_vars()['STORE']
    except KeyError:
        if env_vars()['USE_ROCKSDB'].lower() == 'true':
            store = 'rocksdb://'
        else:
            store = 'memory://'
    return {'store': store}


def get_file_logging_handler():
    if env_vars()['KIWF_LOG_FILEPATH']:
        handler = RotatingFileHandler(env_vars()['KIWF_LOG_FILEPATH'], maxBytes=100000, backupCount=1)
        handler.setLevel(env_vars()['LOGLEVEL'])
        handler.setFormatter(logging.Formatter('[%(asctime)s] [%(process)d] [%(levelname)s] %(message)s'))
        return {'loghandlers': [handler]}
    return {}


def get_config():
    """
    Maps environment variables to Faust app config dictionary
    Any environ.get config options are generally considered optional and generaly default to .
    :return: Faust configs
    :rtype: dict
    """
    config = {
        'id': env_vars()['APP_NAME'],
        'broker': env_vars()['KAFKA_CLUSTER'],
        'topic_partitions': int(env_vars()['TOPIC_PARTITIONS']),
        'topic_replication_factor': int(env_vars()['TOPIC_REPLICATION_FACTOR']),
        'processing_guarantee': env_vars()['PROCESSING_GUARANTEE'],
        'stream_recovery_delay': int(env_vars()['STREAM_RECOVERY_DELAY']),
        'consumer_auto_offset_reset': env_vars()['CONSUMER_AUTO_OFFSET_RESET'],
        'broker_max_poll_records': int(env_vars()['MAX_POLL_RECORDS']),
        'datadir': f"{env_vars()['FAUST_DATADIR_BASE']}/{env_vars()['HOSTNAME']}/"
    }

    config.update(get_data_store())
    config.update(get_ssl_context())
    config.update(get_file_logging_handler())
    return config



from unittest import TestCase
from unittest.mock import MagicMock, patch
from nubium_utils.confluent_utils import produce_message


class TestConfluentProducerUtils(TestCase):

    def setUp(self) -> None:
        self.env_patch = patch.dict(
            'os.environ',
            {"DO_METRICS_PUSHING": "False"}
        )
        self.env_patch.start()
        self.mock_producer = MagicMock()
        self.mock_metrics_manager = MagicMock()

    def tearDown(self) -> None:
        self.env_patch.stop()

    def test_produce_message(self):
        test_input = [
            self.mock_producer,
            dict(topic='test_topic',
                 key='blah_key',
                 value={'blah_value': 'blah_value'},
                 headers={'last_updated_by': 'sfdc'}),
            self.mock_metrics_manager,
            [('guid', b'112233'), ('last_updated_by', b'eloqua')]
        ]

        produce_message(*test_input)
        self.mock_producer.produce.assert_called_with(
            **dict(
                topic='test_topic',
                key='blah_key',
                value={'blah_value': 'blah_value'},
                headers={'guid': '112233', 'last_updated_by': 'sfdc'}))



import setuptools

with open("README.md", "r") as file_obj:
    long_description = file_obj.read()

packages = setuptools.find_packages()

setuptools.setup(
    name='nubium-utils',
    version='0.10.7',
    author="Edward Brennan",
    author_email="ebrennan@redhat.com",
    description="Some Kafka utility functions and patterns for the nubium project",
    long_description=long_description,
    long_description_content_type='text/markdown',
    url="https://gitlab.corp.redhat.com/mkt-ops-de/nubium-utils.git",
    packages=packages,
    install_requires=["requests", "prometheus_client"],
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
)
